import { Injectable } from '@nestjs/common';
import { PassportStrategy } from '@nestjs/passport';
import { JwtPayload } from 'jsonwebtoken';
import { ExtractJwt, Strategy } from 'passport-jwt';
import { AUDIENCE, ISSUER, SECRET } from '../constants';

@Injectable()
export class JwtStrategy extends PassportStrategy(Strategy) {
  constructor() {
    super({
      jwtFromRequest: ExtractJwt.fromAuthHeaderAsBearerToken(),
      ignoreExpiration: false,
      secretOrKey: SECRET,
      audience: AUDIENCE,
      issuer: ISSUER,
    } as ConstructorParameters<typeof Strategy>[0]);
  }

  public async validate(payload: JwtPayload): Promise<{ id: string } | false> {
    return payload.sub ? { id: payload.sub } : false;
  }
}
