import { Field, GraphQLISODateTime, ID, ObjectType } from '@nestjs/graphql';
import {
  Column,
  CreateDateColumn,
  Entity,
  ManyToOne,
  PrimaryGeneratedColumn,
} from 'typeorm';
import { TodoList } from './todoList.model';

@Entity()
@ObjectType()
export class TodoItem {
  @PrimaryGeneratedColumn('uuid')
  @Field(() => ID)
  id: string;

  @Column()
  @Field()
  title: string;

  // DBには保存されない動的に計算するフィールド
  @Field()
  completed: boolean;

  @Column({ type: 'datetime', nullable: true, default: null })
  @Field(() => GraphQLISODateTime, { nullable: true })
  completedAt: Date | null;

  @CreateDateColumn({ type: 'datetime' })
  @Field(() => GraphQLISODateTime)
  createdAt: Date;

  @ManyToOne(() => TodoList, (list) => list.items)
  @Field(() => TodoList)
  list: TodoList;
}
