import { Field, GraphQLISODateTime, ID, ObjectType } from '@nestjs/graphql';
import {
  Column,
  CreateDateColumn,
  Entity,
  ManyToOne,
  OneToMany,
  PrimaryGeneratedColumn,
} from 'typeorm';
import { User } from '../../user/models/user.model';
import { TodoItem } from './todoItem.model';

@Entity()
@ObjectType()
export class TodoList {
  @PrimaryGeneratedColumn('uuid')
  @Field(() => ID)
  id: string;

  @Column()
  @Field()
  name: string;

  @CreateDateColumn({ type: 'datetime' })
  @Field(() => GraphQLISODateTime)
  createdAt: Date;

  @ManyToOne(() => User, (user) => user.todoLists)
  @Field(() => User)
  author: User;

  @OneToMany(() => TodoItem, (item) => item.list)
  @Field(() => [TodoItem])
  items: TodoItem[];
}
