import { InternalServerErrorException } from '@nestjs/common';
import { ResolveField, Resolver } from '@nestjs/graphql';
import { TodoItem } from './models/todoItem.model';
import { TodoList } from './models/todoList.model';
import { TodosService } from './todos.service';

@Resolver(TodoItem)
export class TodoItemResolver {
  constructor(private todosService: TodosService) {}

  @ResolveField()
  async completed(item: TodoItem): Promise<boolean> {
    return item.completedAt !== null;
  }

  @ResolveField()
  async list(item: TodoItem): Promise<TodoList> {
    const found = await this.todosService.getListByItemId(item.id);
    if (!found) {
      throw new InternalServerErrorException();
    }
    return found;
  }
}
