import { Args, Mutation, Query, Resolver } from '@nestjs/graphql';
import { ForbiddenException } from '@nestjs/common';
import { CurrentUser } from '../auth/currentUser.decorator';
import { TodoList } from './models/todoList.model';
import { User } from '../user/models/user.model';
import { TodosService } from './todos.service';
import { CreateListInput } from './dto/createList.input';
import { CreateItemInput } from './dto/createItem.input';
import { CompleteItemInput } from './dto/completeItem.input';
import { TodoItem } from './models/todoItem.model';

@Resolver()
export class TodosResolver {
  constructor(private todosService: TodosService) {}

  @Query(() => TodoList)
  async todoList(@CurrentUser() user: User, @Args('id') todoListId: string) {
    if (!(await this.todosService.isMyList(user.id, todoListId))) {
      throw new ForbiddenException();
    }
    return this.todosService.getListById(todoListId);
  }

  @Mutation(() => TodoList)
  async createList(
    @CurrentUser() user: User,
    @Args('input') input: CreateListInput,
  ) {
    return this.todosService.createList(user.id, input);
  }

  @Mutation(() => TodoItem)
  async createItem(
    @CurrentUser() user: User,
    @Args('input') { todoListId, ...input }: CreateItemInput,
  ) {
    if (!(await this.todosService.isMyList(user.id, todoListId))) {
      throw new ForbiddenException();
    }
    return this.todosService.createItem(todoListId, input);
  }

  @Mutation(() => TodoItem)
  async completeItem(
    @CurrentUser() user: User,
    @Args('input') input: CompleteItemInput,
  ) {
    if (!(await this.todosService.isMyItem(user.id, input.todoItemId))) {
      throw new ForbiddenException();
    }
    return this.todosService.completeItem(input.todoItemId);
  }
}
