import { Injectable } from '@nestjs/common';
import { InjectRepository } from '@nestjs/typeorm';
import { Repository } from 'typeorm';
import { TodoList } from '../todo/models/todoList.model';
import { TodoItem } from './models/todoItem.model';

@Injectable()
export class TodosService {
  constructor(
    @InjectRepository(TodoList)
    private todoListsRepository: Repository<TodoList>,
    @InjectRepository(TodoItem)
    private todoItemsRepository: Repository<TodoItem>,
  ) {}

  async getListById(todoListId: string): Promise<TodoList | null> {
    return this.todoListsRepository.findOne({
      where: { id: todoListId },
    });
  }

  async getListByItemId(todoItemId: string): Promise<TodoList | null> {
    return this.todoListsRepository.findOne({
      where: { items: { id: todoItemId } },
    });
  }

  async getListItems(todoListId: string): Promise<TodoItem[]> {
    const found = await this.todoListsRepository.findOne({
      where: { id: todoListId },
      relations: ['items'],
    });
    return found?.items ?? [];
  }

  async isMyList(userId: string, todoListId: string): Promise<boolean> {
    const list = await this.todoListsRepository.findOne({
      where: { id: todoListId },
      relations: ['author'],
    });
    return list?.author.id === userId;
  }

  async isMyItem(userId: string, todoItemId: string): Promise<boolean> {
    const item = await this.todoItemsRepository.findOne({
      where: { id: todoItemId },
      relations: ['list', 'list.author'],
    });
    return item?.list.author.id === userId;
  }

  async createList(
    userId: string,
    payload: Pick<TodoList, 'name'>,
  ): Promise<TodoList> {
    return this.todoListsRepository.save({
      ...payload,
      author: { id: userId },
    });
  }

  async createItem(
    todoListId: string,
    payload: Partial<TodoItem>,
  ): Promise<TodoItem> {
    return this.todoItemsRepository.save({
      ...payload,
      completedAt: null,
      list: { id: todoListId },
    });
  }

  async completeItem(todoItemId: string): Promise<TodoItem> {
    return this.todoItemsRepository.save({
      id: todoItemId,
      completedAt: new Date(),
    });
  }
}
